@@ -13,7 +13,7 @@ import operator
 import re
 import datetime
 from calendar import isleap
-from decimal import Decimal
+from decimal import Decimal, Context
 from typing import cast, Any, Callable, Dict, Optional, Tuple, Union
 
 from ..helpers import MONTH_DAYS_LEAP, MONTH_DAYS, DAYS_IN_4Y, \
@@ -226,6 +226,15 @@ class AbstractDateTime(metaclass=AtomicTypeMeta):
     def tzinfo(self, tz: Timezone) -> None:
         self._dt = self._dt.replace(tzinfo=tz)
 
+    def tzname(self) -> Optional[str]:
+        return self._dt.tzname()
+
+    def astimezone(self, tz: Optional[datetime.tzinfo] = None) -> datetime.datetime:
+        return self._dt.astimezone(tz)
+
+    def isocalendar(self) -> Tuple[int, int, int]:
+        return self._dt.isocalendar()
+
     @classmethod
     def fromstring(cls, datetime_string: str, tzinfo: Optional[Timezone] = None) \
             -> 'AbstractDateTime':
@@ -302,7 +311,8 @@ class AbstractDateTime(metaclass=AtomicTypeMeta):
     def _get_operands(self, other: object) -> Tuple[datetime.datetime, datetime.datetime]:
         if isinstance(other, (self.__class__, datetime.datetime)) or \
                 isinstance(self, other.__class__):
-            dt = getattr(other, '_dt', other)
+            dt: datetime.datetime = getattr(other, '_dt', cast(datetime.datetime, other))
+
             if self._dt.tzinfo is dt.tzinfo:
                 return self._dt, dt
             elif self.tzinfo is None:
@@ -536,7 +546,7 @@ class DateTime10(OrderedDateTime):
             return '{}-{:02}-{:02}T{:02}:{:02}:{:02}.{}{}'.format(
                 self.iso_year, self.month, self.day, self.hour, self.minute, self.second,
                 '{:06}'.format(self.microsecond).rstrip('0'), str(self.tzinfo or '')
-            ).rstrip('0')
+            )
         return '{}-{:02}-{:02}T{:02}:{:02}:{:02}{}'.format(
             self.iso_year, self.month, self.day, self.hour,
             self.minute, self.second, str(self.tzinfo or '')
@@ -744,7 +754,7 @@ class Duration(AnyAtomicType):
             raise OverflowError("seconds duration overflow")
 
         self.months = months
-        self.seconds = Decimal(seconds).quantize(Decimal('1.000000'))
+        self.seconds = Decimal(seconds).quantize(Decimal('1.000000', context=Context(prec=30)))
 
     def __repr__(self) -> str:
         return '{}(months={!r}, seconds={})'.format(
